/* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */

/* Copyright (C) 2006 Novell, Inc.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 * 02111-1307, USA.
 */

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include "gnome-xgl-settings.h"

#include "eggaccelerators.h"
#include "eggcellrendererkeys.h"

G_DEFINE_TYPE (GXGLKeybinding, gxgl_keybinding, GTK_TYPE_TREE_VIEW);

enum {
	CHANGED,
	LAST_SIGNAL
};

static guint signals[LAST_SIGNAL] = { 0 };

static gboolean button_press_event (GtkWidget *widget, GdkEventButton *event);
static void accel_edited (EggCellRendererKeys *keys, const char *path_string,
                          guint keyval, EggVirtualModifierType mask, guint keycode,
                          gpointer widget);
static void accel_cleared (EggCellRendererKeys *keys, const char *path_string,
                           gpointer widget);

static void
gxgl_keybinding_class_init (GXGLKeybindingClass *keybinding_class)
{
        GObjectClass *object_class = G_OBJECT_CLASS (keybinding_class);
        GtkWidgetClass *widget_class = GTK_WIDGET_CLASS (keybinding_class);

        widget_class->button_press_event = button_press_event;

	signals[CHANGED] = g_signal_new ("changed",
                                         G_OBJECT_CLASS_TYPE (object_class),
                                         G_SIGNAL_RUN_FIRST,
                                         G_STRUCT_OFFSET (GXGLKeybindingClass, changed),
                                         NULL, NULL,
                                         g_cclosure_marshal_VOID__STRING,
                                         G_TYPE_NONE, 1,
                                         G_TYPE_STRING);
}

static void
gxgl_keybinding_init (GXGLKeybinding *keybinding)
{
	GtkListStore *model;
	GtkTreeIter iter;
        GtkCellRenderer *renderer;

        gtk_tree_view_set_headers_visible (GTK_TREE_VIEW (keybinding), FALSE);
        gtk_tree_view_set_enable_search (GTK_TREE_VIEW (keybinding), FALSE);

	model = gtk_list_store_new (3, G_TYPE_UINT, GDK_TYPE_MODIFIER_TYPE, G_TYPE_UINT);
	gtk_list_store_append (model, &iter);
        /* EggCellRendererKeys bug? Passing 0 for the modifiers results in it
         * showing nothing at startup rather than "Disabled".
         */
	gtk_list_store_set (model, &iter, 0, 0, 1, GDK_SHIFT_MASK, 2, 0, -1);

        gtk_tree_view_set_model (GTK_TREE_VIEW (keybinding), GTK_TREE_MODEL (model));

	renderer = g_object_new (EGG_TYPE_CELL_RENDERER_KEYS,
                                 "editable", TRUE,
                                 "accel_mode", EGG_CELL_RENDERER_KEYS_MODE_X,
                                 NULL);
	gtk_tree_view_insert_column_with_attributes (GTK_TREE_VIEW (keybinding),
                                                     0, NULL, renderer,
						     "accel_key", 0,
						     "accel_mask", 1,
						     "keycode", 2,
						     NULL);

	g_signal_connect (renderer, "accel_edited", G_CALLBACK (accel_edited), keybinding);
	g_signal_connect (renderer, "accel_cleared", G_CALLBACK (accel_cleared), keybinding);
}

static gboolean
real_start_editing_cb (gpointer tree_view)
{
  GtkTreePath *path;

  gtk_widget_grab_focus (tree_view);
  path = gtk_tree_path_new_first ();
  gtk_tree_view_set_cursor (tree_view, path, gtk_tree_view_get_column (tree_view, 0), TRUE);
  gtk_tree_path_free (path);

  return FALSE;
}

static gboolean
button_press_event (GtkWidget *widget, GdkEventButton *event)
{
  if (event->window != gtk_tree_view_get_bin_window (GTK_TREE_VIEW (widget)))
          return GTK_WIDGET_CLASS (gxgl_keybinding_parent_class)->button_press_event (widget, event);

  g_signal_stop_emission_by_name (G_OBJECT (widget), "button_press_event");
  g_idle_add (real_start_editing_cb, widget);
  return TRUE;
}

static void
accel_edited (EggCellRendererKeys *keys, const char *path_string,
              guint keyval, EggVirtualModifierType mask, guint keycode,
              gpointer widget)
{
	char *keybinding = egg_virtual_accelerator_name (keyval, keycode, mask);

        g_signal_emit (widget, signals[CHANGED], 0, keybinding);
	g_free (keybinding);

	gtk_tree_selection_unselect_all (gtk_tree_view_get_selection (widget));
}

static void
accel_cleared (EggCellRendererKeys *keys, const char *path_string, gpointer widget)
{
        g_signal_emit (widget, signals[CHANGED], 0, "Disabled");
	gtk_tree_selection_unselect_all (gtk_tree_view_get_selection (widget));
}

GtkWidget *
gxgl_keybinding_new (void)
{
	return g_object_new (GXGL_TYPE_KEYBINDING, NULL);
}


GtkWidget *
gxgl_keybinding_new_glade (const char *string1, const char *string2, int num1, int num2)
{
        GtkWidget *widget = gxgl_keybinding_new ();
	gtk_widget_show (widget);
	return widget;
}

char *
gxgl_keybinding_get (GXGLKeybinding *widget)
{
        GtkTreeModel *model = gtk_tree_view_get_model (GTK_TREE_VIEW (widget));
        guint key, keycode;
	EggVirtualModifierType mods;
	GtkTreeIter iter;

	gtk_tree_model_get_iter_first (model, &iter);
	gtk_tree_model_get (model, &iter, 0, &key, 1, &mods, 2, &keycode, -1);

        return egg_virtual_accelerator_name (key, keycode, mods);
}

void
gxgl_keybinding_set (GXGLKeybinding *widget, const char *binding)
{
        GtkTreeModel *model = gtk_tree_view_get_model (GTK_TREE_VIEW (widget));
        guint key, keycode;
	EggVirtualModifierType mods;
	GtkTreeIter iter;

	if (!egg_accelerator_parse_virtual (binding, &key, &keycode, &mods)) {
		key = keycode = 0;
		mods = 0;
	}

	gtk_tree_model_get_iter_first (model, &iter);
        gtk_list_store_set (GTK_LIST_STORE (model), &iter,
                            0, key, 1, mods, 2, keycode,
                            -1);
}
