/* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */

/* Copyright (C) 2006 Novell, Inc.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 * 02111-1307, USA.
 */

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include <string.h>

#include "gnome-xgl-settings.h"
#ifdef ALL_OPTIONS
#include "gxgl-plugin-options.h"
#endif

#define REQUIRED_PLUGINS "gconf", "decoration", "move", "resize", "place"

static GSList *active_plugins;

typedef struct {
	const char *plugin, *other;
} PluginRelation;

PluginRelation *requires, *supports;

#ifdef ALL_OPTIONS
GHashTable *plugin_options;
#endif

static void
active_plugins_changed (GConfClient *client, guint cnxn_id, GConfEntry *entry, gpointer user_data)
{
	GSList *plugins, *p;
	char *name;
#ifdef ALL_OPTIONS
	GSList *deactivated = NULL, *activated = NULL;
	GXGLPluginOptions *poptions;
#endif

	/* Make sure the required plugins are always enabled */
	if (plugins_activate (REQUIRED_PLUGINS, NULL))
		return;

	plugins = NULL;
	for (p = gconf_value_get_list (entry->value); p; p = p->next) {
		name = g_strdup (gconf_value_get_string (p->data));
		plugins = g_slist_append (plugins, name);
#ifdef ALL_OPTIONS
		if (!g_slist_find_custom (active_plugins, name, (GCompareFunc)strcmp))
			activated = g_slist_prepend (activated, g_strdup (name));
#endif
	}

#ifdef ALL_OPTIONS
	for (p = active_plugins; p; p = p->next) {
		name = p->data;
		if (!g_slist_find_custom (plugins, name, (GCompareFunc)strcmp))
			deactivated = g_slist_prepend (deactivated, g_strdup (name));
	}
#endif

	free_slist (active_plugins);
	active_plugins = plugins;

#ifdef ALL_OPTIONS
	for (p = deactivated; p; p = p->next) {
		poptions = g_hash_table_lookup (plugin_options, p->data);
		if (poptions)
			gxgl_plugin_options_set_active (poptions, FALSE);
	}
	free_slist (deactivated);

	for (p = activated; p; p = p->next) {
		poptions = g_hash_table_lookup (plugin_options, p->data);
		if (poptions)
			gxgl_plugin_options_set_active (poptions, TRUE);
	}
	free_slist (activated);
#endif
}

void
plugins_init (void)
{
	GSList *plugins, *p, *deps, *d;
	const char *dir, *name;
	char *key;
	GArray *req_arr = g_array_new (TRUE, TRUE, sizeof (PluginRelation));
	GArray *supp_arr = g_array_new (TRUE, TRUE, sizeof (PluginRelation));
	PluginRelation rel;
#ifdef ALL_OPTIONS
	GtkWidget *options, *w;
	GtkSizeGroup *size_group = gtk_size_group_new (GTK_SIZE_GROUP_HORIZONTAL);

	plugin_options = g_hash_table_new (g_str_hash, g_str_equal);

	options = gxgl_options_new ("General Compiz Options", NULL,
				    "/apps/compiz/general", size_group);
	w = g_hash_table_lookup (GXGL_OPTIONS (options)->editors, KEY_ACTIVE_PLUGINS);
	if (w)
		gtk_widget_set_sensitive (w, FALSE);
	gtk_box_pack_start (GXGL_WIDGET ("all_options_box"), options, FALSE, FALSE, 0);
#endif

	gconf_client_notify_add (gconf, KEY_ACTIVE_PLUGINS, active_plugins_changed,
				 NULL, NULL, NULL);
	gconf_client_notify (gconf, KEY_ACTIVE_PLUGINS);

	plugins = gconf_client_all_dirs (gconf, "/apps/compiz/plugins", NULL);
	plugins = g_slist_sort (plugins, (GCompareFunc)strcmp);
	for (p = plugins; p; p = p->next) {
		dir = p->data;
		name = strrchr (dir, '/') + 1;

		key = g_strdup_printf ("%s/load_before", dir);
		deps = gconf_client_get_list (gconf, key, GCONF_VALUE_STRING, NULL);
		g_free (key);
		if (deps) {
			for (d = deps; d; d = d->next) {
				rel.plugin = g_strdup (name);
				rel.other = d->data;
				g_array_append_val (supp_arr, rel);
			}
			g_slist_free (deps);
		}

		key = g_strdup_printf ("%s/requires", dir);
		deps = gconf_client_get_list (gconf, key, GCONF_VALUE_STRING, NULL);
		g_free (key);
		if (deps) {
			for (d = deps; d; d = d->next) {
				rel.plugin = g_strdup (name);
				rel.other = d->data;
				g_array_append_val (req_arr, rel);
			}
			g_slist_free (deps);
		}

#ifdef ALL_OPTIONS
		options = gxgl_plugin_options_new (name, size_group);
		g_hash_table_insert (plugin_options, g_strdup (name), options);
		gtk_box_pack_start (GXGL_WIDGET ("all_options_box"), options, FALSE, FALSE, 0);
#endif
	}
	free_slist (plugins);
#ifdef ALL_OPTIONS
	g_object_unref (size_group);
#endif

	requires = (PluginRelation *)g_array_free (req_arr, FALSE);
	supports = (PluginRelation *)g_array_free (supp_arr, FALSE);

	gtk_widget_show_all (GXGL_WIDGET ("all_options_box"));
}

static gboolean
plugin_supports (const char *plugin, const char *other)
{
	int i;
	for (i = 0; supports[i].plugin; i++) {
		if (!strcmp (supports[i].plugin, plugin) &&
		    !strcmp (supports[i].other, other))
			return TRUE;
	}
	return FALSE;
}

static gboolean
plugin_requires (const char *plugin, const char *other)
{
	int i;
	for (i = 0; requires[i].plugin; i++) {
		if (!strcmp (requires[i].plugin, plugin) &&
		    !strcmp (requires[i].other, other))
			return TRUE;
	}
	return FALSE;
}


static void
update_plugins (GSList *plugins)
{
	GSList *p, *q;

	/* This is not efficient, but it will eventually terminate,
	 * assuming "supports" and "requires" contain no loops.
	 */
 start_over:
	for (p = plugins; p; p = p->next) {
		for (q = plugins; q != p; q = q->next) {
			if (plugin_requires (q->data, p->data)) {
				plugins = g_slist_insert_before (plugins, q, p->data);
				plugins = g_slist_delete_link (plugins, p);
				goto start_over;
			}
		}
		for (q = q->next; q; q = q->next) {
			if (plugin_supports (q->data, p->data)) {
				plugins = g_slist_insert_before (plugins, p, q->data);
				plugins = g_slist_delete_link (plugins, q);
				goto start_over;
			}
		}
	}

	gconf_client_set_list (gconf, KEY_ACTIVE_PLUGINS,
			       GCONF_VALUE_STRING, plugins, NULL);
}

gboolean
plugins_activate (const char *plugin, ...)
{
	GSList *plugins;
	gboolean changed = FALSE;
	va_list ap;

	plugins = gconf_client_get_list (gconf, KEY_ACTIVE_PLUGINS,
					 GCONF_VALUE_STRING, NULL);

	va_start (ap, plugin);
	while (plugin) {
		if (!g_slist_find_custom (plugins, plugin, (GCompareFunc)strcmp)) {
			plugins = g_slist_append (plugins, g_strdup (plugin));
			changed = TRUE;
		}
		plugin = va_arg (ap, const char *);
	}
	va_end (ap);

	if (changed)
		update_plugins (plugins);
	free_slist (plugins);
	return changed;
}

gboolean
plugins_deactivate (const char *plugin, ...)
{
	GSList *plugins, *match;
	gboolean changed = FALSE;
	va_list ap;

	plugins = gconf_client_get_list (gconf, KEY_ACTIVE_PLUGINS,
					 GCONF_VALUE_STRING, NULL);

	va_start (ap, plugin);
	while (plugin) {
		match = g_slist_find_custom (plugins, plugin, (GCompareFunc)strcmp);
		if (match) {
			g_free (match->data);
			plugins = g_slist_delete_link (plugins, match);
			changed = TRUE;
		}
		plugin = va_arg (ap, const char *);
	}
	va_end (ap);

	if (changed)
		update_plugins (plugins);
	free_slist (plugins);
	return changed;
}

gboolean
plugin_active (const char *plugin)
{
	GSList *p;

	for (p = active_plugins; p; p = p->next) {
		if (!strcmp (plugin, p->data))
			return TRUE;
	}
	return FALSE;
}

static const char *required_plugins[] = { REQUIRED_PLUGINS };
#define N_REQUIRED_PLUGINS G_N_ELEMENTS (required_plugins)

gboolean
plugin_required (const char *plugin)
{
	int i;

	for (i = 0; i < N_REQUIRED_PLUGINS; i++) {
		if (!strcmp (plugin, required_plugins[i]))
			return TRUE;
	}

	return FALSE;
}

static void
free_string (gpointer string, gpointer data)
{
	g_free (string);
}

void
free_slist (GSList *slist)
{
	g_slist_foreach (slist, free_string, NULL);
	g_slist_free (slist);
}
