/* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */

/* Copyright (C) 2006 Novell, Inc.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 * 02111-1307, USA.
 */

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include "gnome-xgl-settings.h"
#include "eggaccelerators.h"
#include "eggcellrendererkeys.h"

static const struct {
	const char *wname;
	const char *modifier;
	const char *key;
} modifiers[] = {
	{ "shift",   "<Shift>",   "Shift_L" },
	{ "control", "<Control>", "Control_L" },
	{ "alt",     "<Alt>",     "Alt_L" },
	{ "super",   "<Super>",   "Super_L" },
	{ "button1", NULL,        "Button1" },
	{ "button2", NULL,        "Button2" },
	{ "button3", NULL,        "Button3" }
};
static const int N_MODIFIERS = G_N_ELEMENTS (modifiers);

static char *
modifiers_get (const char *base)
{
	GString *mods = g_string_new (NULL);
	int len = strlen (base) + 9, i;
	char *name = g_malloc (len);
	GtkToggleButton *toggle;

	for (i = N_MODIFIERS - 1; i >= 0; i--) {
		snprintf (name, len, "%s_%s", base, modifiers[i].wname);
		toggle = GXGL_WIDGET (name);
		if (toggle && gtk_toggle_button_get_active (toggle)) {
			if (!modifiers[i].modifier)
				g_string_prepend (mods, modifiers[i].key);
			else
				g_string_prepend (mods, modifiers[i].modifier);
		}
	}
	g_free (name);

	if (!mods->len)
		g_string_append (mods, "Disabled");

	return g_string_free (mods, FALSE);
}

static void
modifiers_set (const char *base, const char *mods)
{
	int len = strlen (base) + 9, i;
	char *name = g_malloc (len);
	GtkToggleButton *toggle;
	gboolean active;

	for (i = 0; i < N_MODIFIERS; i++) {
		snprintf (name, len, "%s_%s", base, modifiers[i].wname);
		toggle = GXGL_WIDGET (name);
		if (toggle) {
			active = strstr (mods, modifiers[i].key) != NULL;
			if (!active && modifiers[i].modifier)
				active = strstr (mods, modifiers[i].modifier) != NULL;
			gtk_toggle_button_set_active (toggle, active);
		}
	}
	g_free (name);
}

static void
set_window_types (const char *key, ...)
{
	va_list ap;
	GSList *types = NULL;
	char *item;

	va_start (ap, key);
	do {
		item = va_arg (ap, char *);
		if (item)
			types = g_slist_prepend (types, item);
	} while (item);
	va_end (ap);

	gconf_client_set_list (gconf, key, GCONF_VALUE_STRING, types, NULL);
	g_slist_free (types);
}

static void
free_string (gpointer string, gpointer data)
{
	g_free (string);
}

void
wobbly_windows_toggled (GtkToggleButton *toggle, gpointer user_data)
{
	gboolean active = gtk_toggle_button_get_active (toggle);

	if (active) {
		plugins_activate ("wobbly", NULL);
		gconf_client_set_bool (gconf, KEY_ROTATE_EDGE_FLIP_MOVE, FALSE, NULL);
		set_window_types (KEY_WOBBLY_MOVE_WINDOW_TYPES,
				  "Toolbar", "Menu", "Utility", "Dialog",
				  "ModalDialog", "Normal", NULL);
	} else
		set_window_types (KEY_WOBBLY_MOVE_WINDOW_TYPES, NULL);
}

static void
key_wobbly_move_window_types_changed (GSList *types)
{
	GtkToggleButton *toggle = GXGL_WIDGET ("wobbly_windows");
	gboolean active = gtk_toggle_button_get_active (toggle);

	if (types != NULL) {
		if (!active) {
			gtk_toggle_button_set_active (toggle, TRUE);
			active = TRUE;
		}
	} else if (active) {
		gtk_toggle_button_set_active (toggle, FALSE);
		active = FALSE;
	}

	gtk_widget_set_sensitive (GXGL_WIDGET ("wobbly_snap"), active);
	if (active)
		plugins_activate ("wobbly", NULL);
}

void
wobbly_snap_toggled (GtkToggleButton *toggle, gpointer user_data)
{
	gboolean active = gtk_toggle_button_get_active (toggle);

	if (!gtk_toggle_button_get_active (GXGL_WIDGET ("wobbly_windows")))
		return;

	if (active)
		gconf_client_set_string (gconf, KEY_WOBBLY_SNAP, "<Shift>", NULL);
	else
		gconf_client_set_string (gconf, KEY_WOBBLY_SNAP, "Disabled", NULL);
}

static void
key_wobbly_snap_changed (const char *mods)
{
	GtkToggleButton *toggle = GXGL_WIDGET ("wobbly_snap");
	gboolean active = gtk_toggle_button_get_active (toggle);

	if (!g_ascii_strcasecmp (mods, "<Shift>")) {
		if (!active)
			gtk_toggle_button_set_active (toggle, TRUE);
	} else {
		if (active)
			gtk_toggle_button_set_active (toggle, FALSE);
	}
}

void
translucent_move_toggled (GtkToggleButton *toggle, gpointer user_data)
{
	gboolean active = gtk_toggle_button_get_active (toggle);

	gconf_client_set_int (gconf, KEY_MOVE_OPACITY, active ? 80 : 100, NULL);
}

static void
key_move_opacity_changed (int opacity)
{
	GtkToggleButton *toggle = GXGL_WIDGET ("translucent_move");
	gboolean active = gtk_toggle_button_get_active (toggle);

	if (active && opacity == 100)
		gtk_toggle_button_set_active (toggle, FALSE);
	else if (!active && opacity < 100)
		gtk_toggle_button_set_active (toggle, TRUE);
}

void
wobbly_map_toggled (GtkToggleButton *toggle, gpointer user_data)
{
	gboolean active = gtk_toggle_button_get_active (toggle);

	if (active) {
		plugins_activate ("wobbly", NULL);
		gconf_client_set_string (gconf, KEY_WOBBLY_MAP_EFFECT, "Shiver", NULL);
		set_window_types (KEY_WOBBLY_MAP_WINDOW_TYPES,
				  "Splash", "Dialog", "ModalDialog", "Normal",
				  NULL);
	} else
		set_window_types (KEY_WOBBLY_MAP_WINDOW_TYPES, NULL);
}

static void
key_wobbly_map_effect_changed (const char *effect)
{
	GtkToggleButton *toggle = GXGL_WIDGET ("wobbly_map");
	gboolean active = gtk_toggle_button_get_active (toggle);

	if (!strcmp (effect, "None")) {
		if (active)
			gtk_toggle_button_set_active (toggle, FALSE);
	} else if (!active) {
		GSList *types = gconf_client_get_list (gconf, KEY_WOBBLY_MAP_WINDOW_TYPES,
						       GCONF_VALUE_STRING, NULL);
		if (types) {
			g_slist_foreach (types, free_string, NULL);
			g_slist_free (types);
			gtk_toggle_button_set_active (toggle, TRUE);
			plugins_activate ("wobbly", NULL);
		}
	}
}

static void
key_wobbly_map_window_types_changed (GSList *types)
{
	GtkToggleButton *toggle = GXGL_WIDGET ("wobbly_map");
	gboolean active = gtk_toggle_button_get_active (toggle);

	if (types == NULL) {
		if (active)
			gtk_toggle_button_set_active (toggle, FALSE);
	} else if (!active) {
		char *map_effect = gconf_client_get_string (gconf, KEY_WOBBLY_MAP_EFFECT, NULL);
		if (map_effect && !strcmp (map_effect, "Shiver")) {
			gtk_toggle_button_set_active (toggle, active);
			plugins_activate ("wobbly", NULL);
		}
		g_free (map_effect);
	}
}

#if 0
void
wobbly_focus_toggled (GtkToggleButton *toggle, gpointer user_data)
{
	gboolean active = gtk_toggle_button_get_active (toggle);

	if (active) {
		plugins_activate ("wobbly", NULL);
		gconf_client_set_string (gconf, KEY_WOBBLY_FOCUS_EFFECT, "Shiver", NULL);
		set_window_types (KEY_WOBBLY_FOCUS_WINDOW_TYPES,
				  "Dialog", "ModalDialog", "Normal",
				  NULL);
	} else
		set_window_types (KEY_WOBBLY_FOCUS_WINDOW_TYPES, NULL);
}

static void
key_wobbly_focus_effect_changed (const char *effect)
{
	GtkToggleButton *toggle = GXGL_WIDGET ("wobbly_focus");
	gboolean active = gtk_toggle_button_get_active (toggle);

	if (!strcmp (effect, "None")) {
		if (active)
			gtk_toggle_button_set_active (toggle, FALSE);
	} else if (!active) {
		GSList *types = gconf_client_get_list (gconf, KEY_WOBBLY_FOCUS_WINDOW_TYPES,
						       GCONF_VALUE_STRING, NULL);
		if (types) {
			g_slist_foreach (types, free_string, NULL);
			g_slist_free (types);
			gtk_toggle_button_set_active (toggle, TRUE);
			plugins_activate ("wobbly", NULL);
		}
	}
}

static void
key_wobbly_focus_window_types_changed (GSList *types)
{
	GtkToggleButton *toggle = GXGL_WIDGET ("wobbly_focus");
	gboolean active = gtk_toggle_button_get_active (toggle);

	if (types == NULL) {
		if (active)
			gtk_toggle_button_set_active (toggle, FALSE);
	} else if (!active) {
		char *focus_effect = gconf_client_get_string (gconf, KEY_WOBBLY_FOCUS_EFFECT, NULL);
		if (focus_effect && !strcmp (focus_effect, "Shiver")) {
			gtk_toggle_button_set_active (toggle, active);
			plugins_activate ("wobbly", NULL);
		}
		g_free (focus_effect);
	}
}
#endif

static void
key_active_plugins_wobbly_changed (gboolean active)
{
	GtkToggleButton *toggle;

	if (!active) {
		toggle = GXGL_WIDGET ("wobbly_windows");
		if (active != gtk_toggle_button_get_active (toggle))
			gtk_toggle_button_set_active (toggle, active);
		toggle = GXGL_WIDGET ("wobbly_map");
		if (active != gtk_toggle_button_get_active (toggle))
			gtk_toggle_button_set_active (toggle, active);
#if 0
		toggle = GXGL_WIDGET ("wobbly_focus");
		if (active != gtk_toggle_button_get_active (toggle))
			gtk_toggle_button_set_active (toggle, active);
#endif
	}
}

void
fade_toggled (GtkToggleButton *toggle, gpointer user_data)
{
	if (gtk_toggle_button_get_active (toggle))
		plugins_activate ("fade", NULL);
	else
		plugins_deactivate ("fade", NULL);
}

static void
key_active_plugins_fade_changed (gboolean active)
{
	GtkToggleButton *toggle = GXGL_WIDGET ("fade");

	if (active != gtk_toggle_button_get_active (toggle))
		gtk_toggle_button_set_active (toggle, active);
}

void
minimize_toggled (GtkToggleButton *toggle, gpointer user_data)
{
	if (gtk_toggle_button_get_active (toggle))
		plugins_activate ("minimize", NULL);
	else
		plugins_deactivate ("minimize", NULL);
}

static void
key_active_plugins_minimize_changed (gboolean active)
{
	GtkToggleButton *toggle = GXGL_WIDGET ("minimize");

	if (active != gtk_toggle_button_get_active (toggle))
		gtk_toggle_button_set_active (toggle, active);
}

void
opacity_modifier_toggled (GtkToggleButton *toggle, gpointer user_data)
{
	char *mods, *binding;

	if (!gtk_toggle_button_get_active (GXGL_WIDGET ("opacity")))
		return;

	mods = modifiers_get ("opacity");
	if (g_ascii_strcasecmp (mods, "Disabled") != 0) {
		binding = g_strdup_printf ("%sButton4", mods);
		gconf_client_set_string (gconf, KEY_OPACITY_INCREASE, binding, NULL);
		g_free (binding);
		binding = g_strdup_printf ("%sButton5", mods);
		gconf_client_set_string (gconf, KEY_OPACITY_DECREASE, binding, NULL);
		g_free (binding);
	} else
		modifiers_set ("opacity", "<Alt>");
	g_free (mods);
}

void
opacity_toggled (GtkToggleButton *toggle, gpointer user_data)
{
	gboolean active = gtk_toggle_button_get_active (toggle);

	if (active)
		opacity_modifier_toggled (toggle, user_data);
	else {
		gconf_client_set_string (gconf, KEY_OPACITY_INCREASE, "Disabled", NULL);
		gconf_client_set_string (gconf, KEY_OPACITY_DECREASE, "Disabled", NULL);
	}
}

static void
key_opacity_changed (const char *binding)
{
	GtkToggleButton *toggle = GXGL_WIDGET ("opacity");
	gboolean active = gtk_toggle_button_get_active (toggle);

	char *increase_binding = gconf_client_get_string (gconf, KEY_OPACITY_INCREASE, NULL);
	char *decrease_binding = gconf_client_get_string (gconf, KEY_OPACITY_DECREASE, NULL);
	int modlen = strcspn (increase_binding, "B");

	if (modlen == 0 ||
	    strcmp (increase_binding + modlen, "Button4") != 0 ||
	    strlen (increase_binding) != strlen (decrease_binding) ||
	    strncmp (increase_binding, decrease_binding, modlen) != 0 ||
	    strcmp (decrease_binding + modlen, "Button5") != 0) {
		if (active)
			gtk_toggle_button_set_active (toggle, FALSE);
		gtk_widget_set_sensitive (GXGL_WIDGET ("opacity_modifiers"), FALSE);
		modifiers_set ("opacity", "<Alt>");
		return;
	}

	gtk_widget_set_sensitive (GXGL_WIDGET ("opacity_modifiers"), TRUE);
	modifiers_set ("opacity", increase_binding);
	if (!active)
		gtk_toggle_button_set_active (toggle, TRUE);

	g_free (increase_binding);
	g_free (decrease_binding);
}

void
cube_toggled (GtkToggleButton *toggle, gpointer user_data)
{
	if (gtk_toggle_button_get_active (toggle))
		plugins_activate ("cube", "rotate", "zoom", NULL);
	else
		plugins_deactivate ("cube", "rotate", "zoom", NULL);
}

static void
key_active_plugins_cube_changed (gboolean active)
{
	GtkToggleButton *toggle = GXGL_WIDGET ("cube");

	if (active != gtk_toggle_button_get_active (toggle))
		gtk_toggle_button_set_active (toggle, active);

	gtk_widget_set_sensitive (GXGL_WIDGET ("cube_size"), active);
	gtk_widget_set_sensitive (GXGL_WIDGET ("cube_drag"), active);
	gtk_widget_set_sensitive (GXGL_WIDGET ("cube_drag_modifiers"), active);
	gtk_widget_set_sensitive (GXGL_WIDGET ("cube_keybindings"), active);
	gtk_widget_set_sensitive (GXGL_WIDGET ("edge_flip_section"), active);
	gtk_widget_set_sensitive (GXGL_WIDGET ("zoom_section"), active);
}

void
cube_size_value_changed (GtkSpinButton *spin, gpointer user_data)
{
	int size = (int)gtk_spin_button_get_value (spin);

	gconf_client_set_int (gconf, KEY_CUBE_SIZE, size, NULL);
}

static void cube_keybindings_sides_changed (int sides);

static void
key_cube_size_changed (int size)
{
	GtkSpinButton *spin = GXGL_WIDGET ("cube_size");
	int old_size = (int)gtk_spin_button_get_value (spin);

	if (old_size != size)
		gtk_spin_button_set_value (spin, size);
	cube_keybindings_sides_changed (size);
}

void
cube_drag_modifier_toggled (GtkToggleButton *toggle, gpointer user_data)
{
	char *mods;

	if (!gtk_toggle_button_get_active (GXGL_WIDGET ("cube_drag")))
		return;
	if (GTK_IS_RADIO_BUTTON (toggle) && !gtk_toggle_button_get_active (toggle))
		return;

	mods = modifiers_get ("cube_drag");
	if (!g_ascii_strcasecmp (mods, "Disabled"))
		modifiers_set ("cube_drag", "<Control><Alt>Button1");
	else
		gconf_client_set_string (gconf, KEY_CUBE_DRAG, mods, NULL);
	g_free (mods);
}

void
cube_drag_toggled (GtkToggleButton *toggle, gpointer user_data)
{
	gboolean active = gtk_toggle_button_get_active (toggle);

	if (!gtk_toggle_button_get_active (GXGL_WIDGET ("cube")))
		return;

	if (active)
		cube_drag_modifier_toggled (toggle, user_data);
	else
		gconf_client_set_string (gconf, KEY_CUBE_DRAG, "Disabled", NULL);
}

static void
key_cube_drag_changed (const char *mods)
{
	GtkToggleButton *toggle = GXGL_WIDGET ("cube_drag");
	gboolean active = gtk_toggle_button_get_active (toggle);

	if (!g_ascii_strcasecmp (mods, "Disabled")) {
		gtk_widget_set_sensitive (GXGL_WIDGET ("cube_drag_modifiers"), FALSE);
		if (active)
			gtk_toggle_button_set_active (toggle, FALSE);
		modifiers_set ("cube_drag", "<Control><Alt>Button1");
	} else {
		char *ext_mods = modifiers_get ("cube_drag");

		if (strcmp (mods, ext_mods) != 0)
			modifiers_set ("cube_drag", mods);
		g_free (ext_mods);

		if (!active)
			gtk_toggle_button_set_active (toggle, TRUE);
		gtk_widget_set_sensitive (GXGL_WIDGET ("cube_drag_modifiers"), TRUE);
	}
}

static struct {
	const char *key;
	int minsides;
} cube_keys[] = {
	{ "/apps/compiz/plugins/cube/allscreens/options/unfold_key",                1 },
	{ "/apps/compiz/plugins/rotate/allscreens/options/rotate_left_key",         1 },
	{ "/apps/compiz/plugins/rotate/allscreens/options/rotate_left_window_key",  1 },
	{ "/apps/compiz/plugins/rotate/allscreens/options/rotate_right_key",        1 },
	{ "/apps/compiz/plugins/rotate/allscreens/options/rotate_right_window_key", 1 },
	{ "/apps/compiz/plugins/rotate/allscreens/options/rotate_to_1_key",         1 },
	{ "/apps/compiz/plugins/rotate/allscreens/options/rotate_to_1_window_key",  1 },
	{ "/apps/compiz/plugins/rotate/allscreens/options/rotate_to_2_key",         2 },
	{ "/apps/compiz/plugins/rotate/allscreens/options/rotate_to_2_window_key",  2 },
	{ "/apps/compiz/plugins/rotate/allscreens/options/rotate_to_3_key",         3 },
	{ "/apps/compiz/plugins/rotate/allscreens/options/rotate_to_3_window_key",  3 },
	{ "/apps/compiz/plugins/rotate/allscreens/options/rotate_to_4_key",         4 },
	{ "/apps/compiz/plugins/rotate/allscreens/options/rotate_to_4_window_key",  4 },
	{ "/apps/compiz/plugins/rotate/allscreens/options/rotate_to_5_key",         5 },
	{ "/apps/compiz/plugins/rotate/allscreens/options/rotate_to_5_window_key",  5 },
	{ "/apps/compiz/plugins/rotate/allscreens/options/rotate_to_6_key",         6 },
	{ "/apps/compiz/plugins/rotate/allscreens/options/rotate_to_6_window_key",  6 },
	{ "/apps/compiz/plugins/rotate/allscreens/options/rotate_to_7_key",         7 },
	{ "/apps/compiz/plugins/rotate/allscreens/options/rotate_to_7_window_key",  7 },
	{ "/apps/compiz/plugins/rotate/allscreens/options/rotate_to_8_key",         8 },
	{ "/apps/compiz/plugins/rotate/allscreens/options/rotate_to_8_window_key",  8 },
	{ "/apps/compiz/plugins/rotate/allscreens/options/rotate_to_9_key",         9 },
	{ "/apps/compiz/plugins/rotate/allscreens/options/rotate_to_9_window_key",  9 },
	{ "/apps/compiz/plugins/rotate/allscreens/options/rotate_to_10_key",        10 },
	{ "/apps/compiz/plugins/rotate/allscreens/options/rotate_to_10_window_key", 10 },
	{ "/apps/compiz/plugins/rotate/allscreens/options/rotate_to_11_key",        11 },
	{ "/apps/compiz/plugins/rotate/allscreens/options/rotate_to_11_window_key", 11 },
	{ "/apps/compiz/plugins/rotate/allscreens/options/rotate_to_12_key",        12 },
	{ "/apps/compiz/plugins/rotate/allscreens/options/rotate_to_12_window_key", 12 }
};
#define N_CUBE_KEYS G_N_ELEMENTS (cube_keys)

GtkListStore *cube_keybindings_model;

static void
accel_edited (EggCellRendererKeys *keys, const char *path_string,
	      guint key, EggVirtualModifierType mods, guint keycode,
	      gpointer user_data)
{
	GtkTreeModel *model = GTK_TREE_MODEL (cube_keybindings_model);
	GtkTreePath *path = gtk_tree_path_new_from_string (path_string);
	GtkTreeIter iter;
	const char *gconf_key;
	char *binding;

	gtk_tree_model_get_iter (model, &iter, path);
	gtk_tree_path_free (path);
	gtk_list_store_set (cube_keybindings_model, &iter,
			    2, key,
			    3, keycode,
			    4, mods,
			    -1);

	if (key)
		binding = egg_virtual_accelerator_name (key, keycode, mods);
	else
		binding = g_strdup ("Disabled");

	gtk_tree_model_get (model, &iter,
			    0, &gconf_key,
			    -1);

	gconf_client_set_string (gconf, gconf_key, binding, NULL);
	g_free (binding);
}

static void
accel_cleared (EggCellRendererKeys *keys, const char *path_string, gpointer user_data)
{
	accel_edited (keys, path_string, 0, 0, 0, user_data);
}

static void
setup_cube_keybindings (void)
{
	GtkTreeView *cube_keybindings = GXGL_WIDGET ("cube_keybindings");
	GtkCellRenderer *renderer;
	GtkTreeViewColumn *column;

	cube_keybindings_model =
		gtk_list_store_new (5, G_TYPE_STRING, G_TYPE_STRING,
				    G_TYPE_UINT, GDK_TYPE_MODIFIER_TYPE, G_TYPE_UINT);

	gtk_tree_view_set_model (cube_keybindings, GTK_TREE_MODEL (cube_keybindings_model));

	renderer = gtk_cell_renderer_text_new ();
	column = gtk_tree_view_column_new_with_attributes ("description", renderer,
							   "text", 1,
							   NULL);
	gtk_tree_view_column_set_expand (column, TRUE);
	gtk_tree_view_append_column (cube_keybindings, column);

	renderer = g_object_new (EGG_TYPE_CELL_RENDERER_KEYS,
				 "editable", TRUE,
				 "accel_mode", EGG_CELL_RENDERER_KEYS_MODE_X,
                                 NULL);
	g_signal_connect (renderer, "accel_edited",
			  G_CALLBACK (accel_edited), NULL);
	g_signal_connect (renderer, "accel_cleared",
			  G_CALLBACK (accel_cleared), NULL);
	gtk_tree_view_insert_column_with_attributes (cube_keybindings, -1, "binding", renderer,
						     "accel_key", 2,
						     "keycode", 3,
						     "accel_mask", 4,
						     NULL);
}

static void
cube_keybindings_sides_changed (int sides)
{
	int i;
	char *binding, *schema_key;
	guint key, keycode;
	EggVirtualModifierType mods;
	GConfSchema *schema;
	const char *short_desc;
	GtkTreeIter iter;

	gtk_list_store_clear (cube_keybindings_model);
	for (i = 0; i < N_CUBE_KEYS; i++) {
		if (cube_keys[i].minsides > sides)
			break;

		schema_key = g_strdup_printf ("/schemas%s", cube_keys[i].key);
		schema = gconf_client_get_schema (gconf, schema_key, NULL);
		g_free (schema_key);
		if (!schema)
			continue;
		short_desc = gconf_schema_get_short_desc (schema);

		binding = gconf_client_get_string (gconf, cube_keys[i].key, NULL);
		if (!binding) {
			gconf_schema_free (schema);
			continue;
		}

		if (!egg_accelerator_parse_virtual (binding, &key, &keycode, &mods)) {
			key = keycode = 0;
			mods = 0;
		}

		gtk_list_store_append (cube_keybindings_model, &iter);
		gtk_list_store_set (cube_keybindings_model, &iter,
				    0, cube_keys[i].key,
				    1, short_desc,
				    2, key,
				    3, keycode,
				    4, mods,
				    -1);

		g_free (binding);
		gconf_schema_free (schema);
	}
}

static void
cube_keybindings_binding_changed (const char *gconf_key, const char *binding)
{
	GtkTreeModel *model = GTK_TREE_MODEL (cube_keybindings_model);
	GtkTreeIter iter;
	const char *row_key;
	guint key, keycode;
	EggVirtualModifierType mods;

	if (!gtk_tree_model_get_iter_first (model, &iter))
		return;

	if (!egg_accelerator_parse_virtual (binding, &key, &keycode, &mods)) {
		key = keycode = 0;
		mods = 0;
	}

	do {
		gtk_tree_model_get (model, &iter,
				    0, &row_key,
				    -1);

		if (!strcmp (row_key, gconf_key)) {
			gtk_list_store_set (cube_keybindings_model, &iter,
					    2, key,
					    3, keycode,
					    4, mods,
					    -1);
			break;
		}
	} while (gtk_tree_model_iter_next (model, &iter));
}

void
edge_flip_toggled (GtkToggleButton *toggle, gpointer user_data)
{
	if (!gtk_toggle_button_get_active (toggle))
		return;

	if (toggle == (GtkToggleButton *)GXGL_WIDGET ("edge_flip_disabled")) {
		gconf_client_set_bool (gconf, KEY_ROTATE_EDGE_FLIP_MOVE, FALSE, NULL);
		gconf_client_set_bool (gconf, KEY_ROTATE_EDGE_FLIP_DND, FALSE, NULL);
		gconf_client_set_bool (gconf, KEY_ROTATE_EDGE_FLIP_POINTER, FALSE, NULL);
	} else if (toggle == (GtkToggleButton *)GXGL_WIDGET ("edge_flip_move")) {
		gconf_client_set_bool (gconf, KEY_ROTATE_EDGE_FLIP_POINTER, FALSE, NULL);
		gconf_client_set_bool (gconf, KEY_ROTATE_EDGE_FLIP_MOVE, TRUE, NULL);
		gconf_client_set_bool (gconf, KEY_ROTATE_EDGE_FLIP_DND, TRUE, NULL);
	} else {
		gconf_client_set_bool (gconf, KEY_ROTATE_EDGE_FLIP_MOVE, TRUE, NULL);
		gconf_client_set_bool (gconf, KEY_ROTATE_EDGE_FLIP_DND, TRUE, NULL);
		gconf_client_set_bool (gconf, KEY_ROTATE_EDGE_FLIP_POINTER, TRUE, NULL);
	}
}

static void
key_rotate_edge_flip_changed (const char *key, gboolean val)
{
	GtkToggleButton *edge_flip_disabled = GXGL_WIDGET ("edge_flip_disabled");
	GtkToggleButton *edge_flip_move = GXGL_WIDGET ("edge_flip_move");
	GtkToggleButton *edge_flip_enabled = GXGL_WIDGET ("edge_flip_enabled");

	gboolean flip_pointer = gconf_client_get_bool (gconf, KEY_ROTATE_EDGE_FLIP_POINTER, NULL);
	gboolean flip_move = gconf_client_get_bool (gconf, KEY_ROTATE_EDGE_FLIP_MOVE, NULL);
	gboolean flip_dnd = gconf_client_get_bool (gconf, KEY_ROTATE_EDGE_FLIP_DND, NULL);

	gboolean inconsistent = FALSE;

	if (!flip_pointer && !flip_move && !flip_dnd) {
		if (!gtk_toggle_button_get_active (edge_flip_disabled))
			gtk_toggle_button_set_active (edge_flip_disabled, TRUE);
	} else if (!flip_pointer && flip_move && flip_dnd) {
		if (!gtk_toggle_button_get_active (edge_flip_move))
			gtk_toggle_button_set_active (edge_flip_move, TRUE);
	} else if (flip_pointer && flip_move && flip_dnd) {
		if (!gtk_toggle_button_get_active (edge_flip_enabled))
			gtk_toggle_button_set_active (edge_flip_enabled, TRUE);
	} else
		inconsistent = TRUE;

	gtk_toggle_button_set_inconsistent (edge_flip_disabled, inconsistent);
	gtk_toggle_button_set_inconsistent (edge_flip_move, inconsistent);
	gtk_toggle_button_set_inconsistent (edge_flip_enabled, inconsistent);
	if (inconsistent) {
		/* We need to make all three radio buttons be inactive so that clicking
		 * on any of them will invoke toggled. gtk_toggle_button_set_active()
		 * won't let us deactivate a radio button though.
		 */
		edge_flip_disabled->active = FALSE;
		edge_flip_move->active = FALSE;
		edge_flip_enabled->active = FALSE;
	}
}

void
edge_flip_delay_value_changed (GtkRange *range, gpointer user_data)
{
	gconf_client_set_int (gconf, KEY_ROTATE_FLIP_TIME,
			      (int)gtk_range_get_value (range), NULL);
}

static void
key_rotate_flip_time_changed (int time)
{
	GtkRange *range = GXGL_WIDGET ("edge_flip_delay");

	if (time != (int)gtk_range_get_value (range))
		gtk_range_set_value (range, time);
}

void
scale_key_size_allocate (GtkWidget *widget, GtkAllocation *alloc, gpointer user_data)
{
	/* Because scale_key is set to expand+fill, we can't do this with a size_group. */
	gtk_widget_set_size_request (GXGL_WIDGET ("rain_key"), alloc->width, -1);
}

void
scale_key_changed (GXGLKeybinding *widget, const char *binding)
{
	plugins_activate ("scale", NULL);
	gconf_client_set_string (gconf, KEY_SCALE_INITIATE, binding, NULL);
}

static void
key_scale_initiate_changed (const char *keybinding)
{
	char *old_keybinding = gxgl_keybinding_get (GXGL_WIDGET ("scale_key"));
	if (strcmp (old_keybinding, keybinding) != 0)
		gxgl_keybinding_set (GXGL_WIDGET ("scale_key"), keybinding);
	g_free (old_keybinding);
}

static char *corner_values[] = { "TopLeft", "TopRight", "BottomLeft", "BottomRight" };
static const int N_CORNER_VALUES = G_N_ELEMENTS (corner_values);

void
scale_corner_changed (GtkComboBox *combo, gpointer user_data)
{
	GSList *types;

	if (gtk_toggle_button_get_active (GXGL_WIDGET ("scale_enable_corner"))) {
		types = g_slist_prepend (NULL, corner_values[gtk_combo_box_get_active (combo)]);
		gconf_client_set_list (gconf, KEY_SCALE_CORNERS, GCONF_VALUE_STRING, types, NULL);
		g_slist_free (types);
	}
}

static void
key_scale_corners_changed (GSList *corners)
{
	GtkToggleButton *toggle = GXGL_WIDGET ("scale_enable_corner");
	gboolean active = gtk_toggle_button_get_active (toggle);

	if (corners != NULL) {
		const char *corner = gconf_value_get_string (corners->data);
		GtkComboBox *combo = GXGL_WIDGET ("scale_corner");
		int i;

		for (i = 0; i < N_CORNER_VALUES; i++) {
			if (!strcmp (corner, corner_values[i])) {
				if (gtk_combo_box_get_active (combo) != i)
					gtk_combo_box_set_active (combo, i);
				if (!active)
					gtk_toggle_button_set_active (toggle, TRUE);
				gtk_widget_set_sensitive (GXGL_WIDGET ("scale_corner"), TRUE);
				return;
			}
		}

	}

	if (active)
		gtk_toggle_button_set_active (toggle, FALSE);
	gtk_widget_set_sensitive (GXGL_WIDGET ("scale_corner"), FALSE);
}

void
scale_enable_corner_toggled (GtkToggleButton *toggle, gpointer user_data)
{
	gboolean active = gtk_toggle_button_get_active (toggle);

	if (active) {
		plugins_activate ("scale", NULL);
		scale_corner_changed (GXGL_WIDGET ("scale_corner"), user_data);
	} else
		gconf_client_set_list (gconf, KEY_SCALE_CORNERS, GCONF_VALUE_STRING, NULL, NULL);
}

static void
key_active_plugins_scale_changed (gboolean active)
{
	if (!active) {
		gtk_toggle_button_set_active (GXGL_WIDGET ("scale_enable_corner"), FALSE);
		gxgl_keybinding_set (GXGL_WIDGET ("scale_key"), "Disabled");
	}
}

void
zoom_modifier_toggled (GtkToggleButton *toggle, gpointer user_data)
{
	char *mods;

	if (!gtk_toggle_button_get_active (GXGL_WIDGET ("zoom")))
		return;
	if (GTK_IS_RADIO_BUTTON (toggle) && !gtk_toggle_button_get_active (toggle))
		return;

	mods = modifiers_get ("zoom");
	if (!g_ascii_strcasecmp (mods, "Disabled"))
		modifiers_set ("zoom", "<Super>Button3");
	else
		gconf_client_set_string (gconf, KEY_ZOOM_INITIATE, mods, NULL);
	g_free (mods);
}

void
zoom_toggled (GtkToggleButton *toggle, gpointer user_data)
{
	gboolean active = gtk_toggle_button_get_active (toggle);

	if (active) {
		plugins_activate ("zoom", NULL);
		zoom_modifier_toggled (toggle, user_data);
	} else
		gconf_client_set_string (gconf, KEY_ZOOM_INITIATE, "Disabled", NULL);
}

static void
key_zoom_initiate_changed (const char *mods)
{
	GtkToggleButton *toggle = GXGL_WIDGET ("zoom");
	gboolean active = gtk_toggle_button_get_active (toggle);

	if (!g_ascii_strcasecmp (mods, "Disabled")) {
		gtk_widget_set_sensitive (GXGL_WIDGET ("zoom_modifiers"), FALSE);
		if (active)
			gtk_toggle_button_set_active (toggle, FALSE);
		modifiers_set ("zoom", "<Super>Button3");
	} else {
		char *ext_mods = modifiers_get ("zoom");

		if (strcmp (mods, ext_mods) != 0)
			modifiers_set ("zoom", mods);
		g_free (ext_mods);

		if (!active)
			gtk_toggle_button_set_active (toggle, TRUE);
		gtk_widget_set_sensitive (GXGL_WIDGET ("zoom_modifiers"), TRUE);
	}
}

void
zoom_scroll_modifier_toggled (GtkToggleButton *toggle, gpointer user_data)
{
	char *mods, *binding;

	if (!gtk_toggle_button_get_active (GXGL_WIDGET ("zoom_scroll")))
		return;

	mods = modifiers_get ("zoom_scroll");
	if (g_ascii_strcasecmp (mods, "Disabled") != 0) {
		binding = g_strdup_printf ("%sButton4", mods);
		gconf_client_set_string (gconf, KEY_ZOOM_IN, binding, NULL);
		g_free (binding);
		binding = g_strdup_printf ("%sButton5", mods);
		gconf_client_set_string (gconf, KEY_ZOOM_OUT, binding, NULL);
		g_free (binding);
	} else
		modifiers_set ("zoom_scroll", "<Super>");
	g_free (mods);
}

void
zoom_scroll_toggled (GtkToggleButton *toggle, gpointer user_data)
{
	gboolean active = gtk_toggle_button_get_active (toggle);

	if (active) {
		plugins_activate ("zoom", NULL);
		zoom_scroll_modifier_toggled (toggle, user_data);
	} else {
		gconf_client_set_string (gconf, KEY_ZOOM_IN, "Disabled", NULL);
		gconf_client_set_string (gconf, KEY_ZOOM_OUT, "Disabled", NULL);
	}
}

static void
key_zoom_inout_changed (const char *binding)
{
	GtkToggleButton *toggle = GXGL_WIDGET ("zoom_scroll");
	gboolean active = gtk_toggle_button_get_active (toggle);

	char *in_binding = gconf_client_get_string (gconf, KEY_ZOOM_IN, NULL);
	char *out_binding = gconf_client_get_string (gconf, KEY_ZOOM_OUT, NULL);
	int modlen = strcspn (in_binding, "B");

	if (modlen == 0 ||
	    strcmp (in_binding + modlen, "Button4") != 0 ||
	    strlen (in_binding) != strlen (out_binding) ||
	    strncmp (in_binding, out_binding, modlen) != 0 ||
	    strcmp (out_binding + modlen, "Button5") != 0) {
		if (active)
			gtk_toggle_button_set_active (toggle, FALSE);
		gtk_widget_set_sensitive (GXGL_WIDGET ("zoom_scroll_modifiers"),
					  FALSE);
		modifiers_set ("zoom_scroll", "<Super>");
		return;
	}

	gtk_widget_set_sensitive (GXGL_WIDGET ("zoom_scroll_modifiers"), TRUE);
	modifiers_set ("zoom_scroll", in_binding);
	if (!active)
		gtk_toggle_button_set_active (toggle, TRUE);

	g_free (in_binding);
	g_free (out_binding);
}

static void
key_active_plugins_zoom_changed (gboolean active)
{
	GtkToggleButton *toggle;

	if (!gtk_toggle_button_get_active (GXGL_WIDGET ("cube")))
		return;

	toggle = GXGL_WIDGET ("zoom");
	if (active != gtk_toggle_button_get_active (toggle))
		gtk_toggle_button_set_active (toggle, active);
	toggle = GXGL_WIDGET ("zoom_scroll");
	if (active != gtk_toggle_button_get_active (toggle))
		gtk_toggle_button_set_active (toggle, active);
}

void
water_modifier_toggled (GtkToggleButton *toggle, gpointer user_data)
{
	char *mods;

	if (!gtk_toggle_button_get_active (GXGL_WIDGET ("water")))
		return;

	mods = modifiers_get ("water");
	if (!g_ascii_strcasecmp (mods, "Disabled"))
		modifiers_set ("water", "<Control><Super>");
	else
		gconf_client_set_string (gconf, KEY_WATER_INITIATE, mods, NULL);
	g_free (mods);
}

void
water_toggled (GtkToggleButton *toggle, gpointer user_data)
{
	gboolean active = gtk_toggle_button_get_active (toggle);

	if (active) {
		plugins_activate ("water", NULL);
		water_modifier_toggled (toggle, user_data);
	} else
		gconf_client_set_string (gconf, KEY_WATER_INITIATE, "Disabled", NULL);
}

static void
key_water_initiate_changed (const char *mods)
{
	GtkToggleButton *toggle = GXGL_WIDGET ("water");
	gboolean active = gtk_toggle_button_get_active (toggle);

	if (!g_ascii_strcasecmp (mods, "Disabled")) {
		gtk_widget_set_sensitive (GXGL_WIDGET ("water_modifiers"), FALSE);
		if (active)
			gtk_toggle_button_set_active (toggle, FALSE);
		modifiers_set ("water", "<Control><Super>");
	} else {
		char *ext_mods = modifiers_get ("water");

		if (strcmp (mods, ext_mods) != 0)
			modifiers_set ("water", mods);
		g_free (ext_mods);

		if (!active)
			gtk_toggle_button_set_active (toggle, TRUE);
		gtk_widget_set_sensitive (GXGL_WIDGET ("water_modifiers"), TRUE);
	}
}

void
rain_key_changed (GXGLKeybinding *widget, const char *binding)
{
	plugins_activate ("water", NULL);
	gconf_client_set_string (gconf, KEY_RAIN_TOGGLE, binding, NULL);
}

static void
key_rain_toggle_changed (const char *keybinding)
{
	char *old_keybinding = gxgl_keybinding_get (GXGL_WIDGET ("rain_key"));
	if (strcmp (old_keybinding, keybinding) != 0)
		gxgl_keybinding_set (GXGL_WIDGET ("rain_key"), keybinding);
	g_free (old_keybinding);

	if (!strcmp (keybinding, "Disabled")) {
		gtk_widget_set_sensitive (GXGL_WIDGET ("rain_rate"), FALSE);
	} else {
		gtk_widget_set_sensitive (GXGL_WIDGET ("rain_rate"), TRUE);
		plugins_activate ("water", NULL);
	}
}

void
rain_rate_value_changed (GtkRange *range, gpointer user_data)
{
	int raw_rate = (int)gtk_range_get_value (range), rate;

	if (raw_rate < 50) {
		/* 1 = 10ms to 49 = 490ms */
		rate = raw_rate * 10;
	} else if (raw_rate < 80) {
		/* 50 = .5s to 79 = 15s */
		rate = (raw_rate - 49) * 500;
	} else {
		/* 80 = 30s to 100 = 15.5min */
		rate = (raw_rate - 79) * 30000;
	}
	gconf_client_set_int (gconf, KEY_RAIN_DELAY, rate, NULL);
}

static void
key_rain_delay_changed (int time)
{
	GtkRange *range = GXGL_WIDGET ("rain_rate");
	int rate = gconf_client_get_int (gconf, KEY_RAIN_DELAY, NULL), raw_rate;

	if (rate < 500)
		raw_rate = rate / 10;
	else if (rate < 15000)
		raw_rate = rate / 500 + 49;
	else
		raw_rate = rate / 30000 + 79;

	if (raw_rate != (int)gtk_range_get_value (range))
		gtk_range_set_value (range, raw_rate);
}

static void
key_active_plugins_water_changed (gboolean active)
{
	GtkToggleButton *toggle = GXGL_WIDGET ("water");
	GXGLKeybinding *keybinding = GXGL_WIDGET ("rain_key");

	if (active != gtk_toggle_button_get_active (toggle))
		gtk_toggle_button_set_active (toggle, active);
	if (!active)
		gxgl_keybinding_set (keybinding, "Disabled");
}

static void
key_active_plugins_changed (GSList *plugins)
{
	gboolean saw_fade = FALSE, saw_minimize = FALSE, saw_rotate = FALSE;
	gboolean saw_wobbly = FALSE, saw_scale = FALSE, saw_zoom = FALSE;
	gboolean saw_water = FALSE;
	const char *plugin;

	while (plugins) {
		plugin = gconf_value_get_string (plugins->data);
		if (!strcmp (plugin, "fade"))
			saw_fade = TRUE;
		else if (!strcmp (plugin, "minimize"))
			saw_minimize = TRUE;
		else if (!strcmp (plugin, "rotate"))
			saw_rotate = TRUE;
		else if (!strcmp (plugin, "wobbly"))
			saw_wobbly = TRUE;
		else if (!strcmp (plugin, "scale"))
			saw_scale = TRUE;
		else if (!strcmp (plugin, "zoom"))
			saw_zoom = TRUE;
		else if (!strcmp (plugin, "water"))
			saw_water = TRUE;
		plugins = plugins->next;
	}

	key_active_plugins_fade_changed (saw_fade);
	key_active_plugins_minimize_changed (saw_minimize);
	key_active_plugins_cube_changed (saw_rotate);
	key_active_plugins_wobbly_changed (saw_wobbly);
	key_active_plugins_scale_changed (saw_scale);
	key_active_plugins_zoom_changed (saw_zoom);
	key_active_plugins_water_changed (saw_water);
}

static void
gconf_key_changed (GConfClient *client, guint cnxn_id, GConfEntry *entry, gpointer user_data)
{
	g_return_if_fail (entry->value != NULL);

	if (!strcmp (entry->key, KEY_ACTIVE_PLUGINS))
		key_active_plugins_changed (gconf_value_get_list (entry->value));
	else if (!strcmp (entry->key, KEY_WOBBLY_MOVE_WINDOW_TYPES))
		key_wobbly_move_window_types_changed (gconf_value_get_list (entry->value));
	else if (!strcmp (entry->key, KEY_WOBBLY_SNAP))
		key_wobbly_snap_changed (gconf_value_get_string (entry->value));
	else if (!strcmp (entry->key, KEY_MOVE_OPACITY))
		key_move_opacity_changed (gconf_value_get_int (entry->value));
	else if (!strcmp (entry->key, KEY_WOBBLY_MAP_EFFECT))
		key_wobbly_map_effect_changed (gconf_value_get_string (entry->value));
	else if (!strcmp (entry->key, KEY_WOBBLY_MAP_WINDOW_TYPES))
		key_wobbly_map_window_types_changed (gconf_value_get_list (entry->value));
#if 0
	else if (!strcmp (entry->key, KEY_WOBBLY_FOCUS_EFFECT))
		key_wobbly_focus_effect_changed (gconf_value_get_string (entry->value));
	else if (!strcmp (entry->key, KEY_WOBBLY_FOCUS_WINDOW_TYPES))
		key_wobbly_focus_window_types_changed (gconf_value_get_list (entry->value));
#endif
	else if (!strcmp (entry->key, KEY_OPACITY_INCREASE))
		key_opacity_changed (gconf_value_get_string (entry->value));
	else if (!strcmp (entry->key, KEY_OPACITY_DECREASE))
		key_opacity_changed (gconf_value_get_string (entry->value));
	else if (!strcmp (entry->key, KEY_CUBE_SIZE))
		key_cube_size_changed (gconf_value_get_int (entry->value));
	else if (!strcmp (entry->key, KEY_CUBE_DRAG))
		key_cube_drag_changed (gconf_value_get_string (entry->value));
	else if (!strcmp (entry->key, KEY_ROTATE_EDGE_FLIP_POINTER))
		key_rotate_edge_flip_changed (entry->key, gconf_value_get_bool (entry->value));
	else if (!strcmp (entry->key, KEY_ROTATE_EDGE_FLIP_MOVE))
		key_rotate_edge_flip_changed (entry->key, gconf_value_get_bool (entry->value));
	else if (!strcmp (entry->key, KEY_ROTATE_EDGE_FLIP_DND))
		key_rotate_edge_flip_changed (entry->key, gconf_value_get_bool (entry->value));
	else if (!strcmp (entry->key, KEY_ROTATE_FLIP_TIME))
		key_rotate_flip_time_changed (gconf_value_get_int (entry->value));
	else if (!strcmp (entry->key, KEY_SCALE_INITIATE))
		key_scale_initiate_changed (gconf_value_get_string (entry->value));
	else if (!strcmp (entry->key, KEY_SCALE_CORNERS))
		key_scale_corners_changed (gconf_value_get_list (entry->value));
	else if (!strcmp (entry->key, KEY_ZOOM_INITIATE))
		key_zoom_initiate_changed (gconf_value_get_string (entry->value));
	else if (!strcmp (entry->key, KEY_ZOOM_IN))
		key_zoom_inout_changed (gconf_value_get_string (entry->value));
	else if (!strcmp (entry->key, KEY_ZOOM_OUT))
		key_zoom_inout_changed (gconf_value_get_string (entry->value));
	else if (!strcmp (entry->key, KEY_WATER_INITIATE))
		key_water_initiate_changed (gconf_value_get_string (entry->value));
	else if (!strcmp (entry->key, KEY_RAIN_TOGGLE))
		key_rain_toggle_changed (gconf_value_get_string (entry->value));
	else if (!strcmp (entry->key, KEY_RAIN_DELAY))
		key_rain_delay_changed (gconf_value_get_int (entry->value));
	else if (entry->value->type == GCONF_VALUE_STRING)
		cube_keybindings_binding_changed (entry->key, gconf_value_get_string (entry->value));
}

void
setup_settings_pages (void)
{
	/* glade bug */
	gtk_combo_box_set_active (GXGL_WIDGET ("scale_corner"), 0);

	setup_cube_keybindings ();

	gconf_client_notify_add (gconf, "/apps/compiz", gconf_key_changed, NULL, NULL, NULL);

	gconf_client_notify (gconf, KEY_WOBBLY_MOVE_WINDOW_TYPES);
	gconf_client_notify (gconf, KEY_WOBBLY_SNAP);
	gconf_client_notify (gconf, KEY_MOVE_OPACITY);
	gconf_client_notify (gconf, KEY_WOBBLY_MAP_EFFECT);
	gconf_client_notify (gconf, KEY_WOBBLY_MAP_WINDOW_TYPES);
#if 0
	gconf_client_notify (gconf, KEY_WOBBLY_FOCUS_EFFECT);
	gconf_client_notify (gconf, KEY_WOBBLY_FOCUS_WINDOW_TYPES);
#endif
	gconf_client_notify (gconf, KEY_OPACITY_INCREASE);
	gconf_client_notify (gconf, KEY_OPACITY_DECREASE);
	gconf_client_notify (gconf, KEY_CUBE_SIZE);
	gconf_client_notify (gconf, KEY_CUBE_DRAG);
	gconf_client_notify (gconf, KEY_ROTATE_EDGE_FLIP_POINTER);
	gconf_client_notify (gconf, KEY_ROTATE_FLIP_TIME);
	gconf_client_notify (gconf, KEY_SCALE_INITIATE);
	gconf_client_notify (gconf, KEY_SCALE_CORNERS);
	gconf_client_notify (gconf, KEY_ZOOM_INITIATE);
	gconf_client_notify (gconf, KEY_ZOOM_IN);
	gconf_client_notify (gconf, KEY_ZOOM_OUT);
	gconf_client_notify (gconf, KEY_WATER_INITIATE);
	gconf_client_notify (gconf, KEY_RAIN_TOGGLE);
	gconf_client_notify (gconf, KEY_RAIN_DELAY);
	gconf_client_notify (gconf, KEY_ACTIVE_PLUGINS);
}

/* "Je voudrais avoir écrit une lettre plus courte, mais je n'en ai pas le temps."
 * -- Blaise Pascal
 */
