/* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */

/* Copyright (C) 2006 Novell, Inc.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 * 02111-1307, USA.
 */

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include "gnome-xgl-settings.h"

#include <sys/wait.h>

#include <libgnomeui/gnome-client.h>

static gboolean ran_installer = FALSE, ran_sax = FALSE;

static char card_status = 'U';
static char *card_name = NULL;
static gboolean accel_enabled = FALSE;
static int default_depth = 0;
static char **missing_packages = NULL;
static gboolean xgl_enabled = FALSE;
static char *xgl_opts = NULL;
static char *config_script = NULL;

typedef enum {
	SwitchButtonNone,
	SwitchButtonDisable,
	SwitchButtonEnable,
	SwitchButtonUpdateSettings,
	SwitchButtonInstaller,
	SwitchButtonSaX2,
} ButtonState;
static ButtonState button_state;

static void
update_switch_page (void)
{
	GdkVisual *visual;
	gboolean depth24;
	int width, height;
	gboolean recommended_resolution;
	char *tmp, *comment;
	gboolean free_comment = FALSE;
	const char *supported;
	const char *sax_advice;
	ButtonState sax_button_state;
	GtkButton *button = GXGL_WIDGET ("switch_button");
	GtkNotebook *notebook = GXGL_WIDGET ("notebook");
	GtkTextBuffer *buffer = gtk_text_view_get_buffer (GXGL_WIDGET ("comments"));
	GtkWidget *w;
	int page;

	switch (card_status) {
	case 'G':
		gtk_label_set (GXGL_WIDGET ("video_card_supported"),
			       _("Your graphics card is supported."));
		supported = _("Your graphics card is supported.");
		gtk_widget_hide (GXGL_WIDGET ("video_card_warning_icon"));
		break;
	case 'B':
		gtk_label_set (GXGL_WIDGET ("video_card_supported"),
			       _("Your graphics card will not work with Xgl."));
		supported = _("Your graphics card will not work with Xgl.");
		gtk_widget_show (GXGL_WIDGET ("video_card_warning_icon"));
		break;
	case 'S':
		gtk_label_set (GXGL_WIDGET ("video_card_supported"),
			       _("Your graphics card is not supported, but may work."));
		supported = _("Your graphics card is not supported. It may be able to "
			      "run Xgl, but might be unusably slow.");
		gtk_widget_hide (GXGL_WIDGET ("video_card_warning_icon"));
		break;
	case 'U':
	default:
		gtk_label_set (GXGL_WIDGET ("video_card_supported"),
			       _("Your graphics card is not in Xgl's database."));
		supported = _("Your graphics card is not known to be supported. It might "
			      "or might not support Xgl, and it might or might not be able "
			      "to run Xgl at full speed.");
		gtk_widget_hide (GXGL_WIDGET ("video_card_warning_icon"));
		break;
	}

	tmp = g_strdup_printf ("<b>%s</b>", accel_enabled ? _("Enabled") : _("Disabled"));
	gtk_label_set_markup (GXGL_WIDGET ("accel_status"), tmp);
	g_free (tmp);
	if (accel_enabled)
		gtk_widget_hide (GXGL_WIDGET ("accel_warning_icon"));
	else
		gtk_widget_show (GXGL_WIDGET ("accel_warning_icon"));

	tmp = g_strdup_printf ("<b>%s</b>", xgl_enabled ? (xgl_opts ? _("Enabled, but misconfigured") : _("Enabled")) : _("Disabled"));
	gtk_label_set_markup (GXGL_WIDGET ("xgl_status"), tmp);
	g_free (tmp);
	if (xgl_enabled && xgl_opts)
		gtk_widget_show (GXGL_WIDGET ("xgl_warning_icon"));
	else
		gtk_widget_hide (GXGL_WIDGET ("xgl_warning_icon"));

	gdk_window_get_geometry (gdk_get_default_root_window (), NULL, NULL,
				 &width, &height, NULL);
	recommended_resolution = (width <= 1920 && height <= 1200);

	if (default_depth == 0) {
		visual = gdk_screen_get_system_visual (gdk_screen_get_default ());
		depth24 = visual->depth == 24;
	} else
		depth24 = (default_depth == 24);

	if (ran_sax || (ran_installer && missing_packages)) {
		sax_advice = "";
		sax_button_state = SwitchButtonNone;
	} else if (card_status == 'G' || card_status == 'S') {
		sax_advice = _("You should be able to fix this using the SaX2 configuration "
			       "tool. Press the button below to run SaX2.");
		sax_button_state = SwitchButtonSaX2;
	} else {
		sax_advice = _("You might be able to fix this using the SaX2 configuration "
			       "tool. Press the button below if you'd like to try running SaX2.");
		sax_button_state = SwitchButtonSaX2;
	}

	if (xgl_enabled) {
		if (xgl_opts) {
			comment = _("Xgl is enabled. However, it is not configured with the options "
				    "that are recommended for your video card. Using the wrong "
				    "options may result in slower 3D performance, or might even "
				    "cause Xgl to crash in some cases.\n\nTo fix this, press "
				    "\"Update Desktop Effects Settings\" below. This will log you "
				    "out and return you to the login screen.");
			button_state = SwitchButtonUpdateSettings;
		} else if (card_status != 'G') {
			comment = _("Xgl is enabled, but might not perform well with this card."
				    "\n\nTo disable Xgl and return to the 2D desktop, press "
				    "\"Disable Desktop Effects\" below. This will log you out "
				    "and return you to the login screen.");
			button_state = SwitchButtonDisable;
		} else {
			comment = _("Xgl is enabled.\n\nTo disable it and return to the 2D desktop, "
				    "press \"Disable Desktop Effects\" below. This will log you out "
				    "and return you to the login screen.");
			button_state = SwitchButtonDisable;
		}
	} else if (card_status == 'B') {
		comment = g_strdup_printf ("%s %s", supported, _("Sorry."));
		button_state = SwitchButtonNone;
	} else if (card_status == 'G' || card_status == 'S') {
		if (!accel_enabled && missing_packages && !ran_installer) {
			comment = g_strdup_printf (
				"%s\n\n%s", supported,
				_("However, in order to use Xgl, you need to first install a "
				  "special video driver for your video card. Press the button "
				  "below to run the Software Updater tool to install this "
				  "driver."));
			button_state = SwitchButtonInstaller;
		} else if (!accel_enabled) {
			comment = g_strdup_printf (
				"%s\n\n%s\n\n%s", supported,
				_("However, you do not currently have 3D acceleration enabled."),
				sax_advice);
			button_state = sax_button_state;
		} else if (!depth24) {
			comment = g_strdup_printf (
				"%s\n\n%s\n\n%s", supported,
				_("However, Xgl cannot be enabled, because you do not have "
				  "24-bit color configured."),
				sax_advice);
			button_state = sax_button_state;
		} else {
			comment = g_strdup_printf (
				"%s\n\n%s", supported,
				_("To enable Xgl, press \"Enable Desktop Effects\" below. This will "
				  "log you out and return you to the login screen."));
			button_state = SwitchButtonEnable;
		}
		free_comment = TRUE;
	} else if (accel_enabled) {
		if (!depth24) {
			comment = g_strdup_printf (
				"%s\n\n%s",
				_("Your graphics card is not supported. Furthermore, you "
				  "do not have 24-bit color configured. If you fix this, "
				  "you may be able to run Xgl.\n\n"),
				sax_advice);
			free_comment = TRUE;
			button_state = sax_button_state;
		} else {
			comment = _("Your graphics card is not known to be supported by Xgl. "
				    "However, it does appear to support 3D acceleration.\n\n"
				    "To enable Xgl, press \"Enable Desktop Effects\" below. This "
				    "will log you out and return you to the login screen.\n\n"
				    "If the login screen does not come back up, you will need "
				    "to disable Xgl from the command line by logging in as root "
				    "and running:\n\ngnome-xgl-switch --disable-xgl");
			button_state = SwitchButtonEnable;
		}
	} else if (missing_packages && !ran_installer) {
		/* FIXME: Make this sound less certain once we can break string freeze */
		comment = g_strdup_printf (
			"%s\n\n%s", supported,
			_("However, in order to use Xgl, you need to first install a "
			  "special video driver for your video card. Press the button "
			  "below to run the Software Updater tool to install this "
			  "driver."));
		button_state = SwitchButtonInstaller;
	} else {
		comment = g_strdup_printf (
			"%s\n\n%s",
			_("Your graphics card is not known to be supported by Xgl, and it is "
			  "not currently configured for 3D acceleration. Xgl cannot be "
			  "enabled. However, if 3D acceleration can be enabled on this card, "
			  "then it might be possible to run Xgl."),
			sax_advice);
		free_comment = TRUE;
		button_state = sax_button_state;
	}
	gtk_text_buffer_set_text (buffer, comment, -1);
	if (free_comment)
		g_free (comment);

	if (button_state == SwitchButtonEnable && !recommended_resolution) {
		gtk_text_buffer_insert_at_cursor (buffer, "\n\n", -1);
		gtk_text_buffer_insert_at_cursor (
			buffer, 
			_("NOTE: you have an extremely large screen; if Xgl is too slow "
			  "at this resolution, it may work better at lower resolutions."), -1);
	} if (button_state == SwitchButtonDisable) {
		gtk_text_buffer_insert_at_cursor (buffer, "\n\n", -1);
		gtk_text_buffer_insert_at_cursor (
			buffer, 
			_("You can configure Desktop Effects using the other tabs of " \
			  "this control panel.\n\n" \
			  "If you are seeing bugs caused by Desktop Effects, there may " \
			  "be information about fixes or workarounds at the openSUSE " \
			  "wiki, which you can visit by clicking the \"Help\" button " \
			  "below."), -1);
	} else if (button_state != SwitchButtonNone) {
		gtk_text_buffer_insert_at_cursor (buffer, "\n\n", -1);
		gtk_text_buffer_insert_at_cursor (
			buffer,
			_("If you are having problems configuring Desktop Effects, you can " \
			  "click the \"Help\" button below to visit the troubleshooting " \
			  "page on the openSUSE wiki for advice."), -1);
	}

	switch (button_state) {
	case SwitchButtonDisable:
		gtk_button_set_image (button, gtk_image_new_from_stock (GTK_STOCK_NO,
									GTK_ICON_SIZE_BUTTON));
		gtk_button_set_label (button, _("Disable Desktop Effects"));
		break;

	case SwitchButtonNone:
	case SwitchButtonEnable:
		gtk_button_set_image (button, gtk_image_new_from_stock (GTK_STOCK_YES,
									GTK_ICON_SIZE_BUTTON));
		gtk_button_set_label (button, _("Enable Desktop Effects"));
		break;

	case SwitchButtonUpdateSettings:
		gtk_button_set_image (button, gtk_image_new_from_stock (GTK_STOCK_PREFERENCES,
									GTK_ICON_SIZE_BUTTON));
		gtk_button_set_label (button, _("Update Desktop Effects Settings"));
		break;

	case SwitchButtonInstaller:
		gtk_button_set_image (button,
				      gtk_image_new_from_icon_name ("system-software-update",
								    GTK_ICON_SIZE_BUTTON));
		gtk_button_set_label (button, _("Start Software Updater"));
		break;

	case SwitchButtonSaX2:
		gtk_button_set_image (button,
				      gtk_image_new_from_icon_name ("video-display",
								    GTK_ICON_SIZE_BUTTON));
		gtk_button_set_label (button, _("Start SaX2"));
		break;
	}
	gtk_widget_set_sensitive (GTK_WIDGET (button), button_state != SwitchButtonNone);

	for (page = 1; page < gtk_notebook_get_n_pages (notebook); page++) {
		w = gtk_notebook_get_nth_page (notebook, page);
		gtk_widget_set_sensitive (w, xgl_enabled);
		gtk_widget_set_sensitive (gtk_notebook_get_tab_label (notebook, w), xgl_enabled);
	}
	if (!xgl_enabled)
		gtk_notebook_set_current_page (notebook, 0);
}

void
setup_switch_page (void)
{
	char *xgl_info_out, **lines, *pkgs, *tmp;
	int exit_status;

	lines = NULL;
	if (g_spawn_command_line_sync (BINDIR "/gnome-xgl-info", &xgl_info_out, NULL,
				       &exit_status, NULL)) {
		if (exit_status == 0)
			lines = g_strsplit (xgl_info_out, "\n", 4);
		g_free (xgl_info_out);
	}
	if (!lines || !lines[0] || !lines[1] || !lines[2]) {
		if (lines)
			g_strfreev (lines);
		lines = g_strsplit ("U\n0\n0\n", "\n", 4);
	}

	card_status = lines[0][0];
	if (card_name)
		g_free (card_name);
	card_name = strlen (lines[0]) > 2 ? g_strdup (lines[0] + 2) : g_strdup (_("Unknown"));

	accel_enabled = lines[1][0] == '1';
	default_depth = strtoul (lines[1] + 1, &pkgs, 10);
	if (missing_packages)
		g_strfreev (missing_packages);
	if (*pkgs == ' ')
		pkgs++;
	missing_packages = *pkgs ? g_strsplit (pkgs, " ", 0) : NULL;

	if (config_script) {
		g_free (config_script);
		config_script = NULL;
	}
	if (missing_packages) {
		int npackages = g_strv_length (missing_packages);

		if (*missing_packages[npackages - 1] == '/') {
			config_script = missing_packages[npackages - 1];
			if (npackages == 1) {
				g_free (missing_packages);
				missing_packages = NULL;
			} else
				missing_packages[npackages - 1] = NULL;
		}
	}

	xgl_enabled = lines[2][0] == '1';
	if (xgl_opts)
		g_free (xgl_opts);
	xgl_opts = strlen (lines[2]) > 2 ? g_strdup (lines[2] + 2) : NULL;

	g_strfreev (lines);

	tmp = g_strdup_printf ("<b>%s</b>", card_name);
	gtk_label_set_markup (GXGL_WIDGET ("video_card"), tmp);
	g_free (tmp);
	gtk_label_set_ellipsize (GXGL_WIDGET ("video_card"), PANGO_ELLIPSIZE_END);
	gtk_label_set_max_width_chars (GXGL_WIDGET ("video_card"), 40);

	gtk_tooltips_set_tip (tooltips, GXGL_WIDGET ("video_card_eventbox"),
			      card_name, card_name);

	update_switch_page ();
}

gboolean
eventbox_button_press_event (GtkWidget *widget, GdkEventButton *event, gpointer user_data)
{
	const char *name;

	if (event->type != GDK_2BUTTON_PRESS || !(event->state & GDK_SHIFT_MASK))
		return FALSE;

	name = gtk_widget_get_name (widget);

	if (!strcmp (name, "video_card_supported_eventbox")) {
		char state[2] = { card_status, 0 };
		card_status = "BSUGU"[strcspn ("GBSU", state)];
	} else if (!strcmp (name, "accel_status_eventbox"))
		accel_enabled = !accel_enabled;
	else if (!strcmp (name, "xgl_status_eventbox"))
		xgl_enabled = !xgl_enabled;
	else
		return FALSE;

	update_switch_page ();
	return TRUE;
}

static void
enable_disable (void)
{
	char *argv[5];
	int exit_status;
	GnomeClient *client;

	argv[0] = "/opt/gnome/bin/gnomesu";
	argv[1] = "--";
	argv[2] = BINDIR "/gnome-xgl-switch";
	argv[3] = (button_state == SwitchButtonDisable) ? "--disable-xgl" : "--enable-xgl";
	argv[4] = (button_state == SwitchButtonDisable) ? NULL : xgl_opts;
	argv[5] = NULL;

	if (!g_spawn_sync (NULL, argv, NULL, 0, NULL, NULL,
			   NULL, NULL, &exit_status, NULL) ||
	    exit_status != 0)
		return;

	setup_switch_page ();
	gtk_widget_set_sensitive (GXGL_WIDGET ("notebook"), FALSE);

	/* The logout dialog defaults to the most-recently-used option.
	 * Force it to be "logout", since that's all we want here.
	 */
	gconf_client_set_string (gconf, "/apps/gnome-session/options/logout_option",
				 "logout", NULL);

	/* Request logout */
	client = gnome_master_client ();
	gnome_client_request_save (client, GNOME_SAVE_GLOBAL, TRUE,
				   GNOME_INTERACT_ANY, FALSE, TRUE);
}

static void
spawn_wait_func (GPid pid, int status, gpointer user_data)
{
	gboolean *ran_successfully = user_data;

	g_spawn_close_pid (pid);

	if (ran_successfully) {
		*ran_successfully = WIFEXITED (status) &&
			(WEXITSTATUS (status) == 0);
	}

	gtk_widget_set_sensitive (GXGL_WIDGET ("page_xgl"), TRUE);
	setup_switch_page ();
}

static void
spawn_in_background (char **argv, const char *comment, gboolean *ran_successfully)
{
	GPid pid;

	if (!g_spawn_async (NULL, argv, NULL, G_SPAWN_DO_NOT_REAP_CHILD,
			    NULL, NULL, &pid, NULL)) {
		//FIXME; do we need an error dialog?
		setup_switch_page ();
		return;
	}
	g_child_watch_add (pid, spawn_wait_func, &ran_successfully);

	gtk_widget_set_sensitive (GXGL_WIDGET ("page_xgl"), FALSE);
	gtk_text_buffer_set_text (
		gtk_text_view_get_buffer (GTK_TEXT_VIEW (GXGL_WIDGET ("comments"))),
		comment, -1);
}

static void
run_installer (void)
{
	GPtrArray *argv;
	int i;

	argv = g_ptr_array_new ();
	g_ptr_array_add (argv, "/usr/bin/zen-installer");
	for (i = 0; missing_packages[i]; i++)
		g_ptr_array_add (argv, missing_packages[i]);
	g_ptr_array_add (argv, NULL);

	ran_installer = TRUE;
	spawn_in_background ((char **)argv->pdata,
			     _("Waiting for the Software Update tool to finish..."),
			     NULL);
}

static void
run_sax2 (void)
{
	char *argv[6];

	argv[0] = "/opt/gnome/bin/gnomesu";
	argv[1] = "--";

	if (config_script) {
		argv[2] = config_script;
		argv[3] = NULL;

		ran_sax = TRUE;
	} else {
		argv[2] = "/usr/sbin/sax2";
		argv[3] = "-O";
		argv[4] = "Monitor";
		argv[5] = NULL;
	};

	spawn_in_background (argv, _("Waiting for SaX2 to finish..."),
			     config_script ? NULL : &ran_sax);
}

void
switch_button_clicked (GtkButton *button, gpointer user_data)
{
	switch (button_state) {
	case SwitchButtonEnable:
	case SwitchButtonDisable:
	case SwitchButtonUpdateSettings:
		enable_disable ();
		break;

	case SwitchButtonInstaller:
		run_installer ();
		break;

	case SwitchButtonSaX2:
		run_sax2 ();
		break;

	default:
		break;
	}
}

void
visit_wiki (void)
{
	g_spawn_command_line_sync (
		"/usr/bin/desktop-launch http://en.opensuse.org/Xgl_Troubleshooting",
		NULL, NULL, NULL, NULL);
}
